<?php
/*
  Plugin Name: Preload
  Version: 0.2.1
  Description: Preload next/previous images when in the photo view.
  Plugin URI: http://piwigo.org/ext/extension_view.php?eid=799
  Author: fifreb
  Author URI:
*/

/*
  Preload - A Piwigo Plugin that preloads images for a more responsive browsing experience.
  Copyright (C) 2015 Philippe Troin <phil@fifi.org>

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

defined('PHPWG_PLUGINS_PATH') or die('Hacking attempt!');

define('PRELOAD_ID',       basename(dirname(__FILE__)));
define('PRELOAD_PATH' ,    PHPWG_PLUGINS_PATH . PRELOAD_ID . '/');

if (defined('IN_ADMIN'))
  {
    add_event_handler('get_admin_plugin_menu_links', 'Preload_admin_menu');
  }
else
  {
    add_event_handler('loc_begin_picture', 'Preload_picture_pre_init');
  }

add_event_handler('init', 'Preload_init');

function Preload_init()
{
  global $conf, $template;

  $conf[PRELOAD_ID] = unserialize($conf[PRELOAD_ID]);
  if (!isset($conf[PRELOAD_ID]['imageCount'])) {
    $conf[PRELOAD_ID]['imageCount'] = 1;
  }

  $template->set_prefilter('header', 'Preload_header_prefilter');
}

function Preload_admin_menu($menu)
{
  $menu[] = array(
		  'NAME' => 'Preload',
		  'URL'  => get_root_url() . 'admin.php?page=plugin-' . PRELOAD_ID,
		  );

  return $menu;
}

function Preload_picture_pre_init()
{
  global $template, $pwg_loaded_plugins;

  // Fotorama does its own preloading, do disable the module when in slideshow mode
  if (! (isset($_GET['slideshow']) && isset($pwg_loaded_plugins['Fotorama']))) {
    add_event_handler('loc_begin_page_header', 'Preload_populate_images');
    $template->set_prefilter('picture', 'Preload_picture_prefilter');
    $template->set_prefilter('slideshow', 'Preload_picture_prefilter');
  }
}

function Preload_populate_images()
{
  global $user, $page, $template, $conf, $pwg_loaded_plugins;

  $conf_nb_images = $conf[PRELOAD_ID]['imageCount'];
  if ($conf_nb_images <= 0) {
    return;
  }

  $size = null;
  $size_from_standard = true;
  $is_automatic_size = null;
  if (isset($pwg_loaded_plugins['automatic_size'])) {
    // That's how automatic_size figures out if automatic sizing has been changed by the user
    if (@$_COOKIE['is_automatic_size'] == 'no') {
      $is_automatic_size = false;
    } else {
      $is_automatic_size = true;
      // Thanks for saving that in the template, automatic_size
      $size = $template->smarty->getTemplateVars('automatic_size');
      $size_from_standard = false;
    }
  }
  if ($size_from_standard) {
    $size = pwg_get_session_var('picture_deriv', $conf['derivative_default_size']);
  }

  if ($size == null) {
    error_log("Preload: size is null, is_automatic_size=$is_automatic_size, size_from_standard=$size_from_standard");
    return;
  }

  $ranks = array();
  $min_rank = max(1, $page['current_rank']-$conf_nb_images);
  $max_rank = min($page['last_rank'], $page['current_rank']+$conf_nb_images);
  $next_rank = -1;
  $prev_rank = -1;
  if ($page['current_rank'] > 1) {
    $prev_rank = $page['current_rank']-1;
  }
  if ($page['current_rank'] < $page['last_rank']) {
    $next_rank = $page['current_rank']+1;
  }
  for ($n = $min_rank; $n <= $max_rank; $n++) {
    $ranks[ $page['items'][ $n ]] = $n;
  }

  $query = '
	SELECT *
	  FROM '.IMAGES_TABLE.'
	  WHERE id IN ('.implode(',', array_keys($ranks)).')
	;';

  $result = pwg_query($query);

  $urls = array();
  while ($row = pwg_db_fetch_assoc($result))
    {
      $rank = $ranks[$row['id']];
      $srcImg = new SrcImage($row);
      if ($rank != $page['current_rank']) {
	// error_log("Preloading $size with id ".$row['id']." and rank $rank");
	$urls[] = (new DerivativeImage($size, $srcImg))->get_url();
      }
      if ($user['theme'] == 'elegant') {
	if ($rank != $prev_rank && $rank != $next_rank) {
	  $urls[] = (new DerivativeImage('square', $srcImg))->get_url();
	}
      }
    }

  // Retrieve existing prefetch list
  $curUrls = array();
  if ($is_automatic_size !== true) {
    // The PREFETCH url that's generated with automatic_size is wrong, so ignore it
    $curPrefetch = $template->smarty->getTemplateVars('U_PREFETCH');
    if ($curPrefetch != null) {
      $curUrls[] = $curPrefetch;
    }
  }
  $addUrls = $curUrls;
  foreach ($urls as $url) {
    if (array_search($url, $curUrls) === false) {
      $addUrls[] = $url;
    }
  }
  $template->assign('U_PREFETCH_ARRAY', $addUrls );
}

function Preload_header_prefilter($content, &$smarty)
{
  // Replace the single prefetch entry in the header with a list of prefetch URLs.
  // The template will use U_PREFETCH_ARRAY if set (even if over U_PREFETCH is also set),
  // then fall back to U_PREFETCH.
  return preg_replace('/^([[:space:]]*\{if isset\(\$U_PREFETCH\).*\{\/if\}[[:space:]]*)$/m',
		      '{if is_array($U_PREFETCH_ARRAY)}

{foreach from=$U_PREFETCH_ARRAY item=link key=id}<link rel="prefetch" href="{$link}">
{/foreach}{else}\1{/if}',
		      $content);
}

function Preload_picture_prefilter($content, &$smarty)
{
  // Inject some extra javascript to force prefetching.
  return "{combine_script id='preload' require='jquery' load='header' path='plugins/Preload/js/preload.js'}
".$content;
}

?>
